@@ -52,7 +52,7 @@ var noscriptBM = {
     var allowJS = noscriptUtil.service.getPref("allowURLBarJS", true);
     
     if (isJS && allowJS) {
-      if(noscriptUtil.service.executeJSURL(shortcut, callback)) return;
+      if (noscriptUtil.service.executeJSURL(shortcut, callback)) return;
     } else if (window.getShortcutOrURI && (shortcut.indexOf(" ") > 0  && !isJS || shortcut.indexOf(":") < 0)) {
       var url = getShortcutOrURI(shortcut, {});
       if(jsrx.test(url) && noscriptBM.handleBookmark(url, callback))
@@ -62,10 +62,19 @@ var noscriptBM = {
     callback();
   },
   
-  loadURI: function() { // Fx 3.1 command bar interception
-    return (window.gURLBar && window.gURLBar.handleCommand && window.gURLBar.handleCommand == arguments.callee.caller) 
-           ? noscriptBM.handleURLBarCommand.apply(window, arguments)
-           : noscriptBM.handleURLBarCommandOriginal.apply(window, arguments);
+  loadURI: function() { // Fx 3.5 command bar interception
+    try {
+      if ("gURLBar" in window) {
+        var handleCommand = window.gURLBar.handleCommand;
+        var times = 5;
+        for(var caller, f = arguments.callee; (caller = f.caller) && times; f = caller, times--) {
+          if (caller === handleCommand) {
+            return noscriptBM.handleURLBarCommand.apply(window, arguments);
+          }
+        }
+      }
+    } catch(e) {}
+    return noscriptBM.handleURLBarCommandOriginal.apply(window, arguments);
   },
 
   handleBookmark: function(url, openCallback) {
@@ -75,8 +84,9 @@ var noscriptBM = {
   patchPlacesMethod: function(k, m) {
     if(m in k) {
       // Dirty eval hack due to Tab Mix Plus conflict: http://tmp.garyr.net/forum/viewtopic.php?t=8052
-      k[m] = eval(k[m].toSource()
-        .replace(/\b\w+\.checkURLSecurity\(/, 'noscriptBM.checkURLSecurity('))
+      var src = k[m].toSource();
+      if (!/\bnoscriptBM\b/.test(src))
+        k[m] = eval(src.replace(/\b\w+\.checkURLSecurity\(/, 'noscriptBM.checkURLSecurity('));
     }
   },
   
@@ -103,27 +113,32 @@ var noscriptBM = {
     if(!noscriptUtil.service) return;
     
     // patch bookmark clicks
-    if(window.BookmarksCommand && !noscriptBM.openOneBookmarkOriginal) { 
+    if("BookmarksCommand" in window && !noscriptBM.openOneBookmarkOriginal) { 
       noscriptBM.openOneBookmarkOriginal = BookmarksCommand.openOneBookmark;
       BookmarksCommand.openOneBookmark = noscriptBM.openOneBookmark;
     }
     
     // patch URLBar for keyword-triggered bookmarklets
-    if(window.handleURLBarCommand && !noscriptBM.handleURLBarCommandOriginal) { // Fx 3.0
-      noscriptBM.handleURLBarCommandOriginal = window.handleURLBarCommand;
-      window.handleURLBarCommand = noscriptBM.handleURLBarCommand;
-    } else { // Fx 3.1
-      noscriptBM.handleURLBarCommandOriginal = window.loadURI;
-      window.loadURI = noscriptBM.loadURI;
+    if (!noscriptBM.handleURLBarCommandOriginal) {
+      if("handleURLBarCommand" in window) { // Fx 3.0
+        noscriptBM.handleURLBarCommandOriginal = window.handleURLBarCommand;
+        window.handleURLBarCommand = noscriptBM.handleURLBarCommand;
+      } else { // Fx 3.5
+        noscriptBM.handleURLBarCommandOriginal = window.loadURI;
+        window.loadURI = noscriptBM.loadURI;
+      }
     }
+    
     var pu = window.PlacesUIUtils || window.PlacesUtils || false;
-    if (typeof(pu) == "object" && !noscriptBM.placesUtils) {
+    if (typeof(pu) == "object" && !pu.__noScriptPatch) {
       noscriptBM.placesUtils = pu;
       window.setTimeout(function() {
+        if (pu.__noScriptPatch) return;
+        pu.__noScriptPatch = true;
         var methods = ["openNodeIn", "openSelectedNodeWithEvent"];
-        for each (method in methods)
+        for each (var method in methods)
           noscriptBM.patchPlacesMethod(pu, method);
-      }, 0);
+      }, 50);
     }
   }
 };
