@@ -19,33 +19,41 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 ***** END LICENSE BLOCK *****/
 
+const CC = Components.classes;
+const CI = Components.interfaces;
 
+var ns = noscriptUtil.service;
 
 var nsopt = {
   
-  serv: noscriptUtil.service,
+
   dom2: /^(?:http[s]?|file):\/\/([^\.\?\/#,;:\\\@]+(:?\.[^\.\?\/#,;:\\\@]+$|$))/,
   utils: null,
   init: function() {
-    const ns = this.serv; 
+
     if(ns.uninstalling) { // this should never happen! 
       window.close();
       return;
     }
     
+   
+    
     this.utils = new UIUtils(ns);
     this.utils.resumeTabSelections();
     
-    var locked = this.serv.locked;
-    for each (widget in ["urlText","urlList", "jsglobal", "addButton", "removeButton", "importButton", "exportButton"]) {
-      this[widget] = document.getElementById(widget);
+    abeOpts.init();
+    EF.init();
+    
+    var locked = ns.locked;
+    for each (var widget in ["urlText","urlList", "jsglobal", "addButton", "removeButton", "importButton", "exportButton"]) {
+      this[widget] = $(widget);
       if(locked) this[widget].disabled = true;
     }
      // forbid <a ping>
-    var pingCbx = document.getElementById("mozopt-browser.send_pings");
+    var pingCbx = $("mozopt-browser.send_pings");
     if(pingCbx.getAttribute("label").indexOf("Allow ") == 0) { 
       pingCbx.setAttribute("label", noscriptUtil.getString("allowLocal", ["<a ping...>"]));
-      document.getElementById("opt-noping")
+      $("opt-noping")
               .setAttribute("label", noscriptUtil.getString("forbidLocal", ["<a ping...>"]));
     }
     
@@ -72,15 +80,15 @@ var nsopt = {
       box.value = ns.getPref(prefName);  
     });
     
-    document.getElementById("opt-showPermanent").setAttribute("label", noscriptUtil.getString("allowLocal", ["[...]"]));
-    document.getElementById("opt-showTemp").setAttribute("label", noscriptUtil.getString("allowTemp", ["[...]"]));
-    document.getElementById("opt-showDistrust").setAttribute("label", noscriptUtil.getString("distrust", ["[...]"]));
-    document.getElementById("opt-showGlobal").setAttribute("label", noscriptUtil.getString("allowGlobal"));
+    $("opt-showPermanent").setAttribute("label", noscriptUtil.getString("allowLocal", ["[...]"]));
+    $("opt-showTemp").setAttribute("label", noscriptUtil.getString("allowTemp", ["[...]"]));
+    $("opt-showDistrust").setAttribute("label", noscriptUtil.getString("distrust", ["[...]"]));
+    $("opt-showGlobal").setAttribute("label", noscriptUtil.getString("allowGlobal"));
   
     var notifyHideLabels = noscriptUtil.getString("notifyHide").split("%S");
-    document.getElementById("opt-notify.hide").setAttribute("label", notifyHideLabels[0]);
-    document.getElementById("notifyDelayLabel").setAttribute("value", notifyHideLabels[1]);
-    document.getElementById("notifyDelay").value = ns.getPref("notify.hideDelay", 5);
+    $("opt-notify.hide").setAttribute("label", notifyHideLabels[0]);
+    $("notifyDelayLabel").setAttribute("value", notifyHideLabels[1]);
+    $("notifyDelay").value = ns.getPref("notify.hideDelay", 5);
     
     this.soundChooser.setSample(ns.getPref("sound.block"));
     
@@ -88,16 +96,18 @@ var nsopt = {
     this.toggleGroup = new ConditionalGroup(ns, "toolbarToggle", 3);
     
     var val = ns.getPref("allowHttpsOnly", 0);
-    document.getElementById("sel-allowHttpsOnly").selectedIndex = (val < 0 || val > 2) ? 0 : val;
+    $("sel-allowHttpsOnly").selectedIndex = (val < 0 || val > 2) ? 0 : val;
     
     var shortcut = ns.getPref("keys.toggle");
     if(shortcut) {
       shortcut = shortcut.replace(/VK_([^\.]*).*/g, "$1").replace(/\s+/g, '+').replace(/_/g, ' ');
-      var shortcutLabel = document.getElementById("toolbarToggle-shortcut");
+      var shortcutLabel = $("toolbarToggle-shortcut");
       shortcutLabel.value = "(" + shortcut + ")";
       shortcutLabel.removeAttribute("hidden");
     }
     
+    this.utils.syncGroup($("opt-secureCookies"));
+    
     this.xssEx = new RegExpController(
         "xssEx", 
         ns.rxParsers.multi,
@@ -110,33 +120,71 @@ var nsopt = {
     // hide incompatible options
     if(top.opener && top.opener.noscriptOverlay && !top.opener.noscriptOverlay.getNotificationBox()) {
       // Moz/SeaMonkey, no notifications
-      document.getElementById("fx-notifications").setAttribute("hidden", "true");
+      $("fx-notifications").setAttribute("hidden", "true");
     }
     
     ["clearClick", "opacizeObject"].forEach(function(c) {
       var pref = ns.getPref(c);
-      Array.forEach(document.getElementById(c + "Opts").getElementsByTagName("checkbox"), function(cbx) {        
+      Array.forEach($(c + "Opts").getElementsByTagName("checkbox"), function(cbx) {        
         cbx.setAttribute("checked", !(pref & parseInt(cbx.getAttribute("value"))) ? "false" : "true");
       });
     });
     
     if (!ns.clearClickHandler.isSupported(document))
       ["clearClickOpts", "opt-clearClick.prompt"].forEach(function(id) {
-        document.getElementById(id).setAttribute("hidden", "true"); // Fx <= 1.5
+        $(id).setAttribute("hidden", "true"); // Fx <= 1.5
       });
     
     
-    if (!("nsINavHistoryService" in Components.interfaces)) {
-      document.getElementById("opt-placesPrefs").setAttribute("hidden", "true");
+    if (!ns.placesSupported) {
+      $("opt-placesPrefs").setAttribute("hidden", "true");
     }
     
-    // document.getElementById("policy-tree").view = policyModel;
+    if (ns.canSerializeConf) this.initSerializeButtons();
+    
+    if (ns.smUninstaller) this.initUninstallButton();
+    
+    // $("policy-tree").view = policyModel;
     window.sizeToContent();
     
     this.addButton.setAttribute("enabled", "false");
     this.removeButton.setAttribute("enabled", "false");
   },
   
+  initUninstallButton: function() {
+    this.utils.moveButtonsDown("uninstallButton");
+  },
+  
+  uninstall: function() {
+    ns.smUninstaller.appUninstall(window);
+  },
+  
+  
+  initSerializeButtons: function() {
+    this.utils.moveButtonsDown("importConfButton", "exportConfButton");
+  },
+  
+  
+  importConf: function() {
+    this.chooseFile(
+      this.buttonToTitle("importConfButton"),
+      "Open",
+      function(f) {
+        ns.restoreConf(ns.readFile(f)) && nsopt.reload();
+      }
+    );
+  },
+  exportConf: function() {
+    this.save();
+    this.chooseFile(
+      this.buttonToTitle("exportConfButton"),
+      "Save",
+      function(f) {
+        ns.writeFile(f, ns.serializeConf(true));
+      }
+    );  
+  },
+  
   reset: function() {
     
     if(!noscriptUtil.prompter.confirm(window, 
@@ -144,7 +192,11 @@ var nsopt = {
           noscriptUtil.getString("reset.warning"))
       ) return;
     
-    this.serv.resetDefaults();
+    ns.resetDefaults();
+    this.reload();
+  },
+  
+  reload: function() {
     this.utils.persistTabSelections();
     var op = top.opener;
     if(op && op.noscriptUtil) {
@@ -156,7 +208,7 @@ var nsopt = {
   },
   
   save: function() {
-    const ns = this.serv;
+  
     
     this.utils.visitCheckboxes(
       function(prefName, inverse, checkbox, mozilla) {
@@ -179,20 +231,23 @@ var nsopt = {
       }
     );
     
+    
     this.utils.visitTextboxes(function(prefName, box) {
-      ns.setPref(prefName, box.value);
+      if (box.value != ns.getPref(prefName)) {
+        ns.setPref(prefName, box.value);
+      }
     });
     
     ["clearClick", "opacizeObject"].forEach(function(c) {
       var pref = 0;
-      Array.forEach(document.getElementById(c + "Opts").getElementsByTagName("checkbox"), function(cbx) {
+      Array.forEach($(c + "Opts").getElementsByTagName("checkbox"), function(cbx) {
         if (cbx.checked) pref = pref | parseInt(cbx.getAttribute("value"));
       });
       ns.setPref(c, pref);
     });
     
     
-    ns.setPref("notify.hideDelay", parseInt(document.getElementById("notifyDelay").value) || 
+    ns.setPref("notify.hideDelay", parseInt($("notifyDelay").value) || 
               ns.getPref("notify.hideDelay", 5));
 
     ns.setPref("sound.block", this.soundChooser.getSample());
@@ -200,7 +255,7 @@ var nsopt = {
     this.autoAllowGroup.persist();
     this.toggleGroup.persist();
     
-    ns.setPref("allowHttpsOnly", document.getElementById("sel-allowHttpsOnly").selectedIndex);
+    ns.setPref("allowHttpsOnly", $("sel-allowHttpsOnly").selectedIndex);
     
     var exVal = this.xssEx.getValue();
     if(this.xssEx.validate() || !/\S/.test(exVal)) 
@@ -213,13 +268,15 @@ var nsopt = {
       ns.resetAllowedObjects();
     }
     
+    EF.save();
+    
     var global = this.jsglobal.getAttribute("checked") == "true";
     var untrustedSites = this.untrustedSites;
     var trustedSites = this.trustedSites;
     var tempSites = this.tempSites;
     var gTempSites = this.gTempSites;
     
-    ns.safeCapsOp(function() {
+    ns.safeCapsOp(function(ns) {
       if(ns.untrustedSites.sitesString != untrustedSites.sitesString
           || ns.jsPolicySites.sitesString != trustedSites.sitesString
           || ns.tempSites.sitesString != tempSites.sitesString
@@ -228,6 +285,7 @@ var nsopt = {
         ns.persistUntrusted();
         ns.setPref("temp", tempSites.sitesString);
         ns.setPref("gtemp", gTempSites.sitesString);
+        
         ns.setJSEnabled(trustedSites.sitesList, true, true);
       }
       ns.jsEnabled = global;
@@ -244,16 +302,16 @@ var nsopt = {
       }
     }  
     this.removeButton.setAttribute("disabled", removeDisabled);
-    document.getElementById("revokeButton")
+    $("revokeButton")
       .setAttribute("disabled", this.tempRevoked || 
-          !(this.tempSites.sitesString || this.gTempSites.sitesString || this.serv.objectWhitelistLen));
+          !(this.tempSites.sitesString || this.gTempSites.sitesString || ns.objectWhitelistLen));
     this.urlChanged();
   },
   
   urlChanged: function() {
     var url = this.urlText.value;
     if(url.match(/\s/)) url = this.urlText.value = url.replace(/\s/g,'');
-    var addEnabled = url.length > 0 && (url = this.serv.getSite(url)) ;
+    var addEnabled = url.length > 0 && (url = ns.getSite(url)) ;
     if(addEnabled) {
       var match = url.match(this.dom2);
       if(match) url = match[1];
@@ -270,7 +328,7 @@ var nsopt = {
       if(/\D/.test(txt.value)) txt.value = txt.value.replace(/\D/, "");
     },
     onChange: function(txt) {
-      txt.value = parseInt(txt.value) || noscriptUtil.service.getPref("notify.hideDelay", 5);
+      txt.value = parseInt(txt.value) || ns.getPref("notify.hideDelay", 5);
     }
   },
   
@@ -288,7 +346,6 @@ var nsopt = {
     const policy = this.trustedSites;
     const sites = this.trustedSites.sitesList;
     const ul = this.urlList;
-    const ns = this.serv;
     for(var j = ul.getRowCount(); j-- > 0; ul.removeItemAt(j));
     const dom2 = this.dom2;
     var site, item;
@@ -312,32 +369,26 @@ var nsopt = {
   },
   
   allow: function() {
-    const site = this.serv.getSite(this.urlText.value);
+    const site = ns.getSite(this.urlText.value);
     this.trustedSites.add(site);
     this.tempSites.remove(site, true, true); // see noscriptService#eraseTemp()
     this.gTempSites.remove(site, true, true);
     
-    this.untrustedSites.remove(site, false, !this.serv.mustCascadeTrust(site, false));
+    this.untrustedSites.remove(site, false, !ns.mustCascadeTrust(site, false));
     this.populateUrlList();
     this.ensureVisible(site);
     this.addButton.setAttribute("disabled", "true");
   },
   
   remove: function() {
-    const ns = this.serv;
     const ul = this.urlList;
+    const selectedItems = ul.selectedItems;
     var visIdx = ul.getIndexOfFirstVisibleRow();
     var lastIdx = visIdx + ul.getNumberOfVisibleRows();
-    const selectedItems = ul.selectedItems;
+   
+    
+    
     
-    if(selectedItems.length == 1) {
-      var site = selectedItems[0].value;
-      if(!ns.isMandatory(site)) {
-        ul.removeItemAt(ul.getIndexOfItem(selectedItems[0]));
-        this.trustedSites.remove(site, true);
-      }
-      return;
-    }
     
     var removed = [];
     for(var j = selectedItems.length; j-- > 0;) {
@@ -345,7 +396,20 @@ var nsopt = {
         removed.push(site);
       }
     }
+    if (!removed.length) return;
+    
     this.trustedSites.remove(removed, true); // keepUp
+    this.tempSites.remove(removed, true, true); // see noscriptService#eraseTemp()
+    this.gTempSites.remove(removed, true, true);
+      
+      
+    if(selectedItems.length == 1) {
+      if(removed.length == 1) {
+        ul.removeItemAt(ul.getIndexOfItem(selectedItems[0]));  
+      }
+      return;
+    }
+    
     // TODO: hide flickering
     this.populateUrlList();
     try {
@@ -360,7 +424,6 @@ var nsopt = {
   
   tempRevoked: false,
   revokeTemp: function() {
-    const ns = this.serv;
     this.trustedSites.remove(this.tempSites.sitesList, true, true);
     this.trustedSites.remove(this.gTempSites.sitesList, true, true);
     this.untrustedSites.add(this.gTempSites.sitesList);
@@ -378,21 +441,18 @@ var nsopt = {
         new SoundChooser(
         "sampleURL", 
         this.buttonToTitle("sampleChooseButton"),
-        noscriptUtil.service,
+        ns,
         "chrome://noscript/skin/block.wav"
       ));
   },
   
   
-  importExport: function(op) {
-    const title = this.buttonToTitle(op + "Button");
+  chooseFile: function(title, mode, callback) {
     try {
-      const cc = Components.classes;
-      const ci = Components.interfaces;
-      const IFP = ci.nsIFilePicker;
-      const fp = cc["@mozilla.org/filepicker;1"].createInstance(IFP);
+      const IFP = CI.nsIFilePicker;
+      const fp = CC["@mozilla.org/filepicker;1"].createInstance(IFP);
       
-      fp.init(window,title, op == "import" ? IFP.modeOpen : IFP.modeSave);
+      fp.init(window,title, IFP["mode" + mode]);
       fp.appendFilters(IFP.filterText);
       fp.appendFilters(IFP.filterAll);
       fp.filterIndex = 0;
@@ -400,15 +460,24 @@ var nsopt = {
       const ret = fp.show();
       if(ret == IFP.returnOK || 
           ret == IFP.returnReplace) {
-        this[op + "List"](fp.file);
+        callback.call(nsopt, fp.file);
       }
     } catch(ex) {
       noscriptUtil.prompter.alert(window, title, ex.toString());
     }
   },
   
+  
+  importExport: function(op) {
+    this.chooseFile(
+      this.buttonToTitle(op + "Button"),
+      op == "import" ? "Open" : "Save",
+      this[op + "List"]
+    );
+  },
+  
   importList: function(file) {
-    var all = this.serv.readFile(file).replace(/\s+/g, "\n");
+    var all = ns.readFile(file).replace(/\s+/g, "\n");
     var untrustedPos = all.indexOf("[UNTRUSTED]");
     if(untrustedPos < 0) {
       this.trustedSites.sitesString += "\n" + all;
@@ -422,8 +491,9 @@ var nsopt = {
   },
   
   exportList: function(file) {
-    this.serv.writeFile(file, 
-      this.trustedSites.sitesList.join("\n") + 
+    var list = ns.getPermanentSites(this.trustedSites, this.tempSites);
+    list.remove(ns.mandatorySites.sitesList, true, true);
+    ns.writeFile(file, list.sitesList.join("\n") + 
       "\n[UNTRUSTED]\n" +
       this.untrustedSites.sitesList.join("\n")
     );
@@ -431,7 +501,7 @@ var nsopt = {
   },
   
   syncNsel: function(cbx) {
-    var blockNSWB = document.getElementById("opt-blockNSWB");
+    var blockNSWB = $("opt-blockNSWB");
     if(cbx.checked) {
       blockNSWB.disabled = true;
       blockNSWB.checked = true;
@@ -441,106 +511,352 @@ var nsopt = {
   },
   
   buttonToTitle: function(btid) {
-    return "NoScript - " + document.getElementById(btid).getAttribute("label");
+    return "NoScript - " + $(btid).getAttribute("label");
+  }
+
+}
+
+var ABE = ns.ABE;
+
+var abeOpts = {
+  selectedRS: null,
+  _map: {},
+  errors: false,
+  init: function() {
+    
+    if (!(ABE.legacySupport || ns.Thread.canSpin)) {
+      var tab = $("nsopt-tabABE");
+      if (tab.selected) {
+        tab.parentNode.selectedIndex = 0;
+      }
+      tab.hidden = true;
+      return;
+    }
+    
+    this.list = $("abeRulesets-list");
+    ABE.updateRules();
+    this.populate();
+    window.addEventListener("focus", function(ev) {
+      if (ABE.updateRulesNow()) {
+        abeOpts.populate();
+        if (abeOpts.errors) {
+          abeOpts.errors = false;
+          noscriptUtil.prompter.alert(window, "ABE", ns.getString("ABE.syntaxError"));
+        }
+      }
+    }, false);
+  },
+  
+  _populating: false,
+  populate: function() {
+    this._populating = true;
+    this.errors = false;
+    try {
+      this._map = {};
+      var l = this.list;
+      for(var j = l.getRowCount(); j-- > 0; l.removeItemAt(j));
+      var rulesets = ABE.rulesets;
+      var selItem = null;
+      if (rulesets) {
+        var sel = this.selectedRS && this.selectedRS.name;
+        this.selectedRS = null;
+        var i, name;
+        for each (var rs in rulesets) {
+          name = rs.name;
+          this._map[name] = rs;
+          i = l.appendItem(name, name);
+          if (rs.disabled) i.setAttribute("disabled", "true");
+          if (sel == name) selItem = i;
+          if (rs.errors) {
+            i.className = "noscript-error";
+            this.errors = rs.errors;
+          }
+        }
+      }
+      l.selectedItem = selItem;
+      this.sync();
+    } finally {
+      this._populating = false;
+    }
+  },
+  
+  selected: function(i) {
+    if (!this._populating) this.sync();
+  },
+  
+  select: function(rs) {
+    var name = rs && rs.name;
+    if (!name) return;
+    var l = this.list;
+    if (l.selectedItem && l.selectedItem.value == name) return;
+    
+    for(var j = l.getRowCount(), i; j-- > 0;) {
+      i = l.getItemAtIndex(j);
+      if (i.value == name) {
+        l.selectedItem = i;
+        break;
+      }
+    }
+  },
+  
+  sync: function() {
+    var selItem = this.list.selectedItem;
+   
+    var rs = null;
+    if (selItem) {
+      this.selectedRS = rs = this._map[selItem.value];
+    } else {
+      this.selectedRS = null;
+    }
+    
+    $("abeEnable-button").disabled = ! ($("abeDisable-button").disabled = !rs || rs.disabled);
+    $("abeEdit-button").disabled = !rs || rs.site;
+    $("abeRefresh-button").disabled = this.list.getRowCount() == 0;
+    
+    var text = $("abeRuleset-text");
+    text.className = selItem && selItem.className || '';
+    text.disabled = !selItem || selItem.disabled;
+    text.value = rs && (rs.errors && rs.errors.join("\n\n") || rs.source) || '';
+  },
+  
+  refresh: function() {
+    ABE.refresh();
+    this.populate();
+  },
+  
+  toggle: function(enabled) {
+    var selItem = this.list.selectedItem;
+    var rs = this.selectedRS;
+    if (!(rs && selItem && rs.name == selItem.value)) return;
+    if ((rs.disabled = !enabled)) {
+      selItem.setAttribute("disabled", "true");
+    } else {
+      selItem.removeAttribute("disabled");
+    }
+    ns.setPref("ABE.disabledRulesetNames", ABE.disabledRulesetNames);
+    this.sync();
+  },
+  
+  edit: function(i) {
+    i = i || this.list.selectedItem;
+    if (!i) return;
+    var file = ABE.getRulesetFile(i.value);
+    if (!(file instanceof CI.nsILocalFile)) return;
+   
+    try {
+      file.launch();
+      return;
+    } catch(e) {
+      // probably a *X platform...
+    }
+    
+    var ed = this.editor;
+    if (!ed) return;
+
+    var mimeInfoService = CC["@mozilla.org/uriloader/external-helper-app-service;1"]
+        .getService(CI.nsIMIMEService);
+    var mimeInfo = mimeInfoService
+      .getFromTypeAndExtension( "application/x-abe-rules", "abe" );
+    mimeInfo.preferredAction = mimeInfo.useHelperApp;
+    
+    if ("nsILocalHandlerApp" in CI) {
+      var handler =  CC["@mozilla.org/uriloader/local-handler-app;1"].createInstance(CI.nsILocalHandlerApp);
+      handler.executable = ed;
+      ed = handler;
+    }
+    mimeInfo.preferredApplicationHandler = ed;
+    mimeInfo.launchWithFile(file);      
+  
+  },
+  
+  get editor() {
+    var ed = null;
+    try {
+      ed = ns.prefs.getComplexValue("abe.editor", CI.nsILocalFile);
+      ed.followLinks = true;
+      if (ed.exists() && ed.isExecutable()) return ed;
+      ed = null;
+    } catch(e) {}
+    const IFP = CI.nsIFilePicker;
+    const fp = CC["@mozilla.org/filepicker;1"].createInstance(IFP);
+      
+    fp.init(window, ns.getString("abe.chooseEditor"), IFP.modeOpen);
+    fp.appendFilters(IFP.filterApps);
+    fp.filterIndex = 0;
+    const ret = fp.show();
+    if (ret == IFP.returnOK) {
+      ed = fp.file;
+      if (ed.exists() && ed.isExecutable()) {  
+        ns.prefs.setComplexValue("abe.editor", CI.nsILocalFile, ed);
+      } else ed = null;
+    }
+    return ed;
   }
   
+}
+
+var EF = {
+  _dirty: false,
+  _compare: function(a, b) { return a.name > b.name ? 1 : a.name < b.name ? -1 : 0; },
+  currentFilter: null,
+  get list() {
+    delete this.list;
+    return this.list = $("ef-list");
+  },
+  get filters() {
+    delete this.filters;
+    return this.filters = ns.externalFilters.cloneFilters();
+  },
   
-  
-}
-
-/*
-function Site(url, perm, temp, disabled) {
-  this.url = url;
-  this.perm = perm;
-  this.temp = temp;
-  this.disabled = disabled;
-}
-
-Site.prototype = {
-  get status() { return this.perm ? "TRUSTED" : "UNTRUSTED" }
-}
-
-Site.sort = function(array, field, descending) {
-  array.sort(function(a,b) {
-    var res;
-    if(field in a) {
-      if(field in b) {
-        a = a[field]; b = b[field];
-        res =a < b ? -1 : a > b ? 1 : 0;
-      } else {
-        res = 1;
+  init: function() {
+    var tab = $("nsopt-tabEF");
+    if (!ns.externalFilters.supported) {
+      if (tab.selected) {
+        tab.parentNode.selectedIndex = 0;
       }
+      tab.hidden = true;
+      return;
+    }
+    
+    if (tab.selected) {
+      this.populate();
     } else {
-      res=(field in b)? -1 : 0;
+      tab.addEventListener("command", function(ev) {
+        tab.removeEventListener("command", arguments.callee, false);
+        EF.populate();
+      }, false);
     }
-    return descending ? -res : res;
-  });
-  this.currentSorting.field = field;
-  this.currentSorting.descending = descending;
-  return array;
+  },
+  
+  populate: function() {
+    const ef = ns.externalFilters;
+    const list = this.list;
+    const filters = this.filters;
+    filters.sort(this._compare);
+    list.removeAllItems();
+    list.selectedItem = null;
+    
+    const filterName = ef.lastFilterName || filters[0] && filters[0].name; 
+    
+    filters.forEach(function(f) {
+      var mi = list.appendItem(f.name);
+      if (!f.valid) mi.setAttribute("class", "noscript-error");
+      if (f.name == filterName) {
+        list.selectedItem = mi;
+        $("ef-exe").value = f.exe && f.exe.path || '';
+        $("ef-type").value = f.contentType;
+        $("ef-exceptions").value = f.whitelist && f.whitelist.source || '';
+        this.currentFilter = f;
+      }
+    }, this);
+    
+    ["ef-remove", "ef-browse", "ef-type", "ef-exceptions"].map($).forEach(
+      this.currentFilter
+      ? function(el) { el && el.removeAttribute("disabled"); }
+      : function(el) { el && el.setAttribute("disabled", "true"); }
+    );
+    
+    if (this.currentFilter) {
+      if (this.currentFilter.builtIn) {
+        $("ef-remove").setAttribute("disabled", "true");
+      } else {
+        $("ef-remove").removeAttribute("disabled");
+      }
+    }
+    this.validate();
+  },
+  
+  onSelect: function(ev) {
+    ns.externalFilters.lastFilterName = this.list.selectedItem && this.list.selectedItem.label;
+    this.populate();
+  },
+  
+  onTypeChange: function(ev) {
+    if (!this.currentFilter) return;
+    this.currentFilter.contentType = ev.target.value;
+    this._dirty = true;
+  },
+  
+  onExceptionsChange: function(ev) {
+    if (this.currentFilter) {
+      var node = ev.target;
+      var wl = new ns.AddressMatcher(node.value);
+      if (wl.rx) {
+        node.removeAttribute("class");
+        this.currentFilter.whitelist = wl;
+        this._dirty = true;
+      } else {
+        node.setAttribute("class", "noscript-error");
+      }
+    }
+  },
+  
+  locateExe: function(f) {
+    f = f || this.currentFilter;
+    if (!f) return;
+    
+    const IFP = CI.nsIFilePicker;
+    const fp = CC["@mozilla.org/filepicker;1"].createInstance(IFP);
+      
+    fp.init(window, ns.getString("ef.locateExe", [f.name]), IFP.modeOpen);
+    fp.appendFilters(IFP.filterApps);
+    fp.filterIndex = 0;
+    const ret = fp.show();
+    if (ret == IFP.returnOK) {
+      var exe = fp.file;
+      if (exe.exists() && exe.isExecutable()) {  
+        f.exe = exe;
+        $("ef-exe").value = exe.path;
+        this._dirty = true;
+      }
+    }
+    this.validate();
+  },
+  
+  validate: function() {
+    var mi = this.list.selectedItem;
+    if (mi) {
+      if (this.currentFilter && !this.currentFilter.valid && this.list.selectedItem) {
+        mi.setAttribute("class", "noscript-error");
+      } else {
+        mi.removeAttribute("class");
+        return true;
+      }
+    }
+    return false;
+  },
+  
+  create: function() {
+    const ret = { value: "" };
+    if(noscriptUtil.prompter.prompt(window, document.title,
+          ns.getString("ef.newName"), ret, null, {}) && 
+       /^[a-z]/i.test(ret.value)
+       ) {
+      
+      const ef = ns.externalFilters;
+      var f = ef.create(ret.value);
+      if (!f.name) return;
+      
+      if (!this.filters.some(f.same, f)) {
+        this.locateExe(f);
+        if (!f.exe) return;
+        this.filters.push(f);
+      }
+      
+      ef.lastFilterName = f.name;
+      this.populate();
+    }
+  },
+  
+  remove: function() {
+    if (this.currentFilter &&
+        noscriptUtil.prompter.confirm(window, document.title, ns.getString("confirm"))) {
+      this.filters.splice(this.filter.indexOf(this.currentFilter), 1);
+      this.populate();
+    }
+  },
+  
+  save: function() {
+    if (this._dirty) ns.externalFilters.save(this.filters);
+  }
 }
-
-Site.currentSorting = { field: 'url', descending: false };
-
-var policyModel = {
-  data: [],
-  get rowCount() { return this.data.length;  },
-  
-  _getColName: function(col) {
-    if(!col) return null;
-    const id=(col && col.id)?col.id:col;
-    var pos = id.lastIndexOf("-");
-    return id.substring(pos + 1);
-  },
-  getCellText: function(row, col) {
-    var colName = this._getColName(col);
-    if(!colName) return "";
-    return this.data[row][colName];
-  },
-  //setCellText: function(row, column, text) {},
-  setTree: function(treeBox) { this.treeBox = treeBox; },
-  isContainer: function(index) { return false; },
-  isSeparator: function(index) { return false; }, 
-  isSorted: function() {},
-  getLevel: function(index) { return 0; },
-  getImageSrc: function(row, col) {
-   return null;
-  },
-  getCellProperties: function(row, col, props) {},
-  getColumnProperties: function(column, elem, prop) {}, 
-  getRowProperties: function(row, props) { },
-
-  isContainerOpen: function(index) { },
-  isContainerEmpty: function(index) { return false; },
-  canDropOn: function(index) { return false; },
-  canDropBeforeAfter: function(index, before) { return false; },
-  drop: function(row, orientation) { return false; },
-  
-  getParentIndex: function(index) { return 0; },
-  hasNextSibling: function(index, after) { return false; },
-  getProgressMode: function(row, column) { },
-  getCellValue: function(row, column) { },
-  toggleOpenState: function(index) { },
-  cycleHeader: function(col, elem) { 
-    if(!elem) {
-      elem=col.element;
-    }
-    const colName = this._getColName(col);
-    const descending = elem.getAttribute("sortDirection") == "ascending";
-    elem.setAttribute("sortDirection", descending ? "descending" : "ascending");
-    Sitesort(this.data, colName, descending);
-    this.treeBox.invalidate();
-  },
-  selectionChanged: function() {  
-    this._messageSelected(this.data[this.selection.currentIndex]);
-  },
-  
-  cycleCell: function(row, column) { },
-  isEditable: function(row, column) { return false; },
-  performAction: function(action) { },
-  performActionOnRow: function(action, row) { },
-  performActionOnCell: function(action, row, column) { }
-};
-*/
-
-
